/*

  This file is part of EASYMOVE.DLL source code, see MAIN.C for
  detailed information.

  Copyright (C) 2000-2001 Pavel Vitis
  pavel.vitis@seznam.cz
  http://lsg.come.to/


  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

/****************************************************************************
****************************************************************************/

#include <windows.h>
#include <memory.h>
#include "main.h"
#include "lsapi.h"
#include "hookdll.h"

CONFIG config;
char *steprcPath = NULL;

// Get ULONG number from string, set parsing_error and display
// error message if some error occured.
// Somewhat multi-purpose function, not too healthy C technique...
void errorBox( LPCTSTR hs, LPCTSTR prm )
{
	char msgbuf[256];

	sprintf(msgbuf, "stroke: \"%s\"\x0Dparameter: \"%s\"\x0DUnknown parameter", hs, prm);
	MessageBox( hDesktop, msgbuf, ST_NAME " parsing error", MB_OK | MB_ICONEXCLAMATION | MB_TOPMOST );
}

// Build list of excluded windows
void loadExcludedWindows(void)
{
	FILE * f;
	unsigned int i;
	PTSTR token[4];
	char buffer[MAX_LINE_LENGTH];
	if (config.ex_win_list)
		free(config.ex_win_list);
	config.ex_win_list = NULL;
	config.ex_win_cnt = 0;

	f = LCOpen(NULL);
	if (!f)
		return;
	for (i = 0; i < 4; i++)
		token[i] = malloc(MAX_LINE_LENGTH);
	while (LCReadNextConfig (f, "*EasymoveExcludeWin", buffer, sizeof(buffer)))
	{
		int count;
		count = LCTokenize (buffer, token, 4, NULL);
		if (count == 4)
		{
			if (!config.ex_win_list)
				config.ex_win_list = (EX_WIN *)malloc(sizeof(EX_WIN));
			else
				config.ex_win_list = (EX_WIN *)realloc(config.ex_win_list, (config.ex_win_cnt+1)*sizeof(EX_WIN));

			int length = strlen(token[1]);
			config.ex_win_list[config.ex_win_cnt].module = malloc(length+1);
			strncpy(config.ex_win_list[config.ex_win_cnt].module, token[1], length+1);
			config.ex_win_list[config.ex_win_cnt].module[length] = 0;

			length = strlen(token[2]);
			config.ex_win_list[config.ex_win_cnt].class = malloc(length+1);
			strncpy(config.ex_win_list[config.ex_win_cnt].class, token[2], length+1);
			config.ex_win_list[config.ex_win_cnt].class[length] = 0;

			length = strlen(token[3]);
			config.ex_win_list[config.ex_win_cnt].title = malloc(length+1);
			strncpy(config.ex_win_list[config.ex_win_cnt].title, token[3], length+1);
			config.ex_win_list[config.ex_win_cnt].title[length] = 0;

			LOG("Window exclusion loaded:");
			LOGF("  module: '%s'", config.ex_win_list[config.ex_win_cnt].module);
			LOGF("  class : '%s'", config.ex_win_list[config.ex_win_cnt].class);
			LOGF("  title : '%s'", config.ex_win_list[config.ex_win_cnt].title);

			config.ex_win_cnt++;
		}
	}
	LCClose(f);
	for (i = 0; i < 3; i++)
		free(token[i]);
	return;
}


BOOL loadConfig(void)
{
	char buffer[MAX_LINE_LENGTH];

	config.enabled = TRUE;

	// Set mouse button that will activate easymove
	config.mbutton = MK_LBUTTON;
	switch ( GetRCInt("EasymoveButton", 1) )
	{
		case 0:
			config.mbutton = -1;	// No mouse button. Hotkey should be set in this case.
			break;
		case 1:
			config.mbutton = MK_LBUTTON;
			break;
		case 2:
			config.mbutton = MK_RBUTTON;
			break;
		case 3:
			config.mbutton = MK_MBUTTON;
			break;
		case 4:
			config.mbutton = MK_XBUTTON1;
			break;
		case 5:
			config.mbutton = MK_XBUTTON2;
			break;
	}
	LOGF("mbutton: '%s'", config.mbutton == MK_LBUTTON ? "LEFT" :
						  config.mbutton == MK_RBUTTON ? "RIGHT" :
						  config.mbutton == MK_MBUTTON ? "MIDDLE" :
						  config.mbutton == -1 ? "NONE" :
						  "UNKNOWN VALUE");

	// Set hotkey.
	config.hotkey = 0;
	if ( GetRCString("EasymoveHotkey", buffer, "", sizeof(buffer)) )
	{
		strupr(buffer);
		LOGF("hotkey: '%s'", buffer);

		if (strstr(buffer, "CAPSLOCK"))
			config.hotkey = VK_CAPITAL;
		else
/*		if (strstr(buffer, "LWIN"))
		{
			if ((LWinKey = GlobalAddAtom("LWIN_KEY")) != 0)
			{
				if (!RegisterHotKey(ToolWindow, LWinKey, MOD_WIN, VK_LWIN))
				{
					GlobalDeleteAtom(LWinKey);
					LWinKey = 0;
				}
			}
			if (LWinKey == 0)
			{
				LOG("Error registering Left WinKey!");
			}
		}
		else
		if (strstr(buffer, "RWIN"))
		{
			if ((RWinKey = GlobalAddAtom("RWIN_KEY")) != 0)
			{
				if (!RegisterHotKey(ToolWindow, RWinKey, MOD_WIN, VK_RWIN))
				{
					GlobalDeleteAtom(RWinKey);
					RWinKey = 0;
				}
			}
			if (RWinKey == 0)
			{
				LOG("Error registering Right WinKey!");
			}
		}
		else
*/		{
			if (strstr(buffer, "SHIFT"))
				config.hotkey |= HK_SHIFT;
			if (strstr(buffer, "ALT"))
				config.hotkey |= HK_ALT;
			if (strstr(buffer, "CTRL"))
				config.hotkey |= HK_CTRL;
			if (strstr(buffer, "CTRL"))
				config.hotkey |= HK_CTRL;
		}

		LOGF("hotkey is '%x'", config.hotkey);
	}


	// Allocate sound paths
	if ( GetRCString("EasymoveSound", buffer, "", sizeof(buffer)) )
		config.def_snd = strdup(buffer);
	else
		config.def_snd = NULL;

	// Load list of *StrokeExcludeWin from step.rc. Build ex_win_list array.
	loadExcludedWindows();

	return TRUE;
}

BOOL reloadConfig(void)
{
	freeConfig();
	if (!loadConfig())
		return FALSE;
	hook_init();

	return TRUE;
}

void freeConfig(void)
{
	if ( config.def_snd )
		free( config.def_snd );
	if (config.ex_win_list)
		free( config.ex_win_list );
}

void hook_init(void)
{
	setMouseHook( ToolWindow,
//		EV_MOUSEWHEEL | EV_MOUSEMOVE | EV_NCMOUSEMOVE |
		(config.mbutton == MK_LBUTTON ? EV_LBUTTONDOWN /*| EV_LBUTTONUP /*| EV_NCLBUTTONDOWN | EV_NCLBUTTONUP*/ : // Left button
		 config.mbutton == MK_RBUTTON ? EV_RBUTTONDOWN /*| EV_RBUTTONUP | EV_NCRBUTTONDOWN | EV_NCRBUTTONUP*/ : // Right button (default)
		 config.mbutton == MK_MBUTTON ? EV_MBUTTONDOWN /*| EV_MBUTTONUP | EV_NCMBUTTONDOWN | EV_NCMBUTTONUP*/ : // Middle button
		 config.mbutton == MK_XBUTTON1 ? EV_XBUTTONDOWN : // Middle button
		 config.mbutton == MK_XBUTTON2 ? EV_XBUTTONDOWN : // Middle button
		 0 // check no buttons
		)
	);

	// Set keyboard-hook only if standalone hotkey is used. If not, hotkey is tested if mouse-button
	// is pressed, so no keyboard hook is needed.
	if ( (config.hotkey != 0)/* && (config.mbutton == -1)*/ )
	{
		LOG("Keyboard hook has been set!");
		setKeyboardHook( ToolWindow, EV_KEYDOWN | EV_KEYUP );
	}
}
