#ifndef  __LSAPI_H
#define __LSAPI_H

#include <windows.h>
#include <stdio.h>
#include "../lsapi/wharfdata.h"

/* match defines */
#define MATCH_PATTERN  6    /* bad pattern */
#define MATCH_LITERAL  5    /* match failure on literal match */
#define MATCH_RANGE    4    /* match failure on [..] construct */
#define MATCH_ABORT    3    /* premature end of text string */
#define MATCH_END      2    /* premature end of pattern string */
#define MATCH_VALID    1    /* valid match */

/* pattern defines */
#define PATTERN_VALID  0    /* valid pattern */
#define PATTERN_ESC   -1    /* literal escape at end of pattern */
#define PATTERN_RANGE -2    /* malformed range in [..] construct */
#define PATTERN_CLOSE -3    /* no end bracket in [..] construct */
#define PATTERN_EMPTY -4    /* [..] contstruct is empty */

/* message defines */
#define LM_SHUTDOWN          8889
#define LM_REPAINT           8890
#define LM_BRINGTOFRONT      8891
#define LM_SAVEDATA			 8892
#define LM_RESTOREDATA		 8893
#define LM_POPUP             9182
#define LM_HIDEPOPUP         9183
#define LM_FIRSTDESKTOPPAINT 9184
#define LM_LSSELECT          9185
#define LM_SETTASKBARONTOP   9186
#define LM_SAVESYSTRAY       9210
#define LM_RESTORESYSTRAY    9211
#define LM_CHECKFORAPPBAR    9212
#define	LM_SENDSYSTRAY		 9213
#define LM_SYSTRAY           9214
#define LM_ADDWINDOW		 9220
#define LM_REMOVEWINDOW		 9221
#define LM_MINMAXWIN		 9222
#define LM_ACTIVEWIN		 9223
#define LM_RECYCLE           9260
#define LM_REGISTERMESSAGE   9263
#define LM_UNREGISTERMESSAGE 9264
#define LM_GETREVID			 9265
#define LM_UNLOADMODULE		 9266
#define LM_RELOADMODULE		 9267
#define LM_SHADETOGGLE       9300
#define LM_REFRESH			 9305
#define LM_VWMUP             9350
#define LM_VWMDOWN           9351
#define LM_VWMLEFT           9352
#define LM_VWMRIGHT          9353
#define LM_VWMNAV            9354
#define LM_SWITCHTON         9355
#define LM_WINLIST           9400

#define LM_WINDOWCREATED		9501
#define LM_WINDOWDESTROYED		9502
#define LM_ACTIVATESHELLWINDOW	9503
#define LM_WINDOWACTIVATED		9504
#define LM_GETMINRECT			9505
#define LM_REDRAW				9506
#define LM_TASKMAN				9507
#define LM_LANGUAGE				9508
#define LM_ACCESSIBILITYSTATE	9511
#define LM_APPCOMMAND			9512

#define MAX_LINE_LENGTH		4096
#define MAX_PATH_LENGTH		1024

#define MAX_BANGCOMMAND		64
#define MAX_BANGARGS		256


#ifdef	__cplusplus
extern "C" {
#endif	/* __cplusplus */

__declspec(dllimport) FILE* LCOpen (LPCTSTR szPath);
__declspec(dllimport) BOOL LCClose (FILE *f);
__declspec(dllimport) BOOL LCReadNextCommand (FILE *f, LPSTR szBuffer, DWORD dwLength);
__declspec(dllimport) BOOL LCReadNextConfig (FILE *f, LPCSTR szPrefix, LPSTR szBuffer, DWORD dwLength);
__declspec(dllimport) BOOL LCReadNextLine (FILE *f, LPSTR szBuffer, DWORD dwLength);
__declspec(dllimport) int LCTokenize (LPCSTR szString, char **lpszBuffers, DWORD dwNumBuffers, LPSTR szExtraParameters);
__declspec(dllimport) HRGN BitmapToRegion(HBITMAP hBmp, COLORREF cTransparentColor, COLORREF cTolerance, int xoffset, int yoffset);
__declspec(dllimport) BOOL ParseBangCommand (HWND caller, char *command, char *args);
__declspec(dllimport) BOOL AddBangCommand(char *command, void * f);
__declspec(dllimport) BOOL AddBangCommandEx(char *command, void * f);
__declspec(dllimport) BOOL RemoveBangCommand(char *command);
__declspec(dllimport) int CommandTokenize (LPCSTR szString, LPSTR *lpszBuffers, DWORD dwNumBuffers, LPSTR szExtraParameters);
__declspec(dllimport) HINSTANCE LSExecute(HWND Owner, LPCSTR szCommand, int nShowCmd);
__declspec(dllimport) HINSTANCE LSExecuteEx(HWND Owner, LPCSTR szOperation, LPCSTR szCommand, LPCSTR szArgs, LPCSTR szDirectory, int nShowCmd);
__declspec(dllimport) HBITMAP LoadLSImage(LPCSTR szImage, LPCSTR szFile);
__declspec(dllimport) HBITMAP BitmapFromIcon (HICON hIcon);
__declspec(dllimport) HICON LoadLSIcon (LPCSTR szImage, LPCSTR szFile);
__declspec(dllimport) void SetDesktopArea(int left, int top, int right, int bottom);
__declspec(dllimport) void GetLSBitmapSize(HBITMAP hBitmap, int *x, int *y);
__declspec(dllimport) void TransparentBltLS (HDC dc, int nXDest, int nYDest, int nWidth, int nHeight, HDC tempDC, int nXSrc, int nYSrc, COLORREF colorTransparent);
__declspec(dllimport) HWND GetLitestepWnd();
__declspec(dllimport) void VarExpansion(char *buffer, const char * value);
__declspec(dllimport) BOOL SetupRC(LPCTSTR szPath);
__declspec(dllimport) int GetRCInt(LPCTSTR lpKeyName, int nDefault);
__declspec(dllimport) BOOL GetRCString(LPCTSTR lpKeyName, LPSTR value, LPCTSTR defStr, int maxLen);
__declspec(dllimport) BOOL GetRCBool(LPCTSTR lpKeyName, BOOL ifFound);
__declspec(dllimport) BOOL GetRCLine( LPCTSTR, LPTSTR, UINT, LPCTSTR );
__declspec(dllimport) COLORREF GetRCColor(LPCTSTR lpKeyName, COLORREF colDef);
__declspec(dllimport) void CloseRC(void);
__declspec(dllimport) void Frame3D(HDC dc, RECT rect, COLORREF TopColor, COLORREF BottomColor, int Width);
__declspec(dllimport) BOOL match (char *pattern, char *text);
__declspec(dllimport) int matche(char *pattern, char *text);
__declspec(dllimport) BOOL is_pattern (char *pattern);
__declspec(dllimport) BOOL is_valid_pattern (char *pattern, int *error_type);

#ifdef	__cplusplus
};
#endif	/* __cplusplus */

#endif  /* __LSAPI_H */
